@@ -1,10 +1,11 @@
+import os
 import pickle
 
 from abc import abstractmethod, abstractclassmethod, ABC
-from pandas import DataFrame
+from pandas import DataFrame, Series
+from pathlib import Path
 
 import numpy
-import pkg_resources
 
 from .metadata import __copyright__, __license__, __version__
 
@@ -19,6 +20,34 @@ def _canonicalizeAll(arguments_df, nan_as_missing):
 		arguments_df = arguments_df.replace({numpy.nan: None})
 	return arguments_df
 
+class PythonEvaluatorUtil:
+	JAVA_CLASS_NAME = "org.jpmml.evaluator.python.PythonEvaluatorUtil"
+
+	def __init__(self):
+		raise RuntimeError()
+
+	@staticmethod
+	def evaluate(backend, javaEvaluator, arguments, dropColumns):
+		arguments = backend.dumps(arguments)
+		dropColumns = backend.newArray("java.lang.String", dropColumns) if dropColumns else None
+		try:
+			results = backend.staticInvoke(PythonEvaluatorUtil.JAVA_CLASS_NAME, "evaluate", javaEvaluator, arguments, dropColumns)
+		except Exception as e:
+			raise backend.toJavaError(e)
+		results = backend.loads(results)
+		return results
+
+	@staticmethod
+	def evaluateAll(backend, javaEvaluator, arguments_dict, dropColumns, parallelism):
+		arguments_dict = backend.dumps(arguments_dict)
+		dropColumns = backend.newArray("java.lang.String", dropColumns) if dropColumns else None
+		try:
+			results_dict = backend.staticInvoke(PythonEvaluatorUtil.JAVA_CLASS_NAME, "evaluateAll", javaEvaluator, arguments_dict, dropColumns, parallelism)
+		except Exception as e:
+			raise backend.toJavaError(e)
+		results_dict = backend.loads(results_dict)
+		return results_dict
+
 class JavaBackend(ABC):
 
 	def __init__(self):
@@ -42,6 +71,10 @@ class JavaBackend(ABC):
 	def _loadJavaClass(self, className):
 		raise NotImplementedError()
 
+	@abstractmethod
+	def newArray(self, className, values):
+		raise NotImplementedError()
+
 	@abstractmethod
 	def newObject(self, className, *args):
 		raise NotImplementedError()
@@ -149,34 +182,38 @@ class Evaluator(JavaObject):
 
 	def evaluate(self, arguments, nan_as_missing = True):
 		arguments = _canonicalize(arguments, nan_as_missing = nan_as_missing)
-		arguments = self.backend.dumps(arguments)
-		try:
-			results = self.backend.staticInvoke("org.jpmml.evaluator.python.PythonUtil", "evaluate", self.javaEvaluator, arguments)
-		except Exception as e:
-			raise self.backend.toJavaError(e)
-		results = self.backend.loads(results)
-		if hasattr(self, "dropColumns"):
-			for dropColumn in self.dropColumns:
-				del results[dropColumn]
+		results = PythonEvaluatorUtil.evaluate(self.backend, self.javaEvaluator, arguments, self.dropColumns if hasattr(self, "dropColumns") else None)
 		return results
 
-	def evaluateAll(self, arguments_df, nan_as_missing = True):
+	def evaluateAll(self, arguments_df, nan_as_missing = True, error_col = "errors", parallelism = -1):
 		arguments_df = _canonicalizeAll(arguments_df, nan_as_missing = nan_as_missing)
+		columns = arguments_df.columns.tolist()
+		data = []
+		for column in columns:
+			data.append(arguments_df[column].tolist())
 		arguments_dict = {
-			"columns" : arguments_df.columns.tolist(),
-			"data" : arguments_df.values.tolist()
+			"columns" : columns,
+			"data" : data
 		}
-		arguments = self.backend.dumps(arguments_dict)
-		try:
-			results = self.backend.staticInvoke("org.jpmml.evaluator.python.PythonUtil", "evaluateAll", self.javaEvaluator, arguments)
-		except Exception as e:
-			raise self.backend.toJavaError(e)
-		results_dict = self.backend.loads(results)
-		results_df = DataFrame(data = results_dict["data"], index = (arguments_df.index.copy() if (len(arguments_dict["data"]) == len(results_dict["data"])) else None), columns = results_dict["columns"])
-		if hasattr(self, "dropColumns"):
-			for dropColumn in self.dropColumns:
-				results_df.drop(str(dropColumn), axis = 1, inplace = True)
-		return results_df
+		results_dict = PythonEvaluatorUtil.evaluateAll(self.backend, self.javaEvaluator, arguments_dict, self.dropColumns if hasattr(self, "dropColumns") else None, parallelism)
+		columns = results_dict["columns"]
+		data = results_dict["data"]
+		errors = results_dict["errors"]
+		results_df = DataFrame()
+		for idx, column in enumerate(columns):
+			results_df[column] = data[idx]
+		if len(arguments_df) == len(results_df):
+			results_df.index = arguments_df.index.copy()
+		if errors is not None:
+			errors = Series(errors, name = error_col, dtype = str)
+			if len(arguments_df) == len(errors):
+				errors.index = arguments_df.index.copy()
+		if error_col:
+			if errors is not None:
+				results_df[error_col] = errors
+			return results_df
+		else:
+			return (results_df, errors)
 
 	def predict(self, X):
 		return self.evaluateAll(X)
@@ -228,8 +265,19 @@ class LoadingModelEvaluatorBuilder(BaseModelEvaluatorBuilder):
 		return self
 
 	def loadFile(self, path):
-		file = self.backend.newObject("java.io.File", path)
-		self.javaModelEvaluatorBuilder.load(file)
+		if isinstance(path, Path):
+			path = str(path)
+		javaFile = self.backend.newObject("java.io.File", path)
+		self.javaModelEvaluatorBuilder.load(javaFile)
+		return self
+
+	def loadString(self, string):
+		return self.loadBytes(bytes(string, "utf-8"))
+
+	def loadBytes(self, bytes):
+		javaInputStream = self.backend.newObject("java.io.ByteArrayInputStream", bytes)
+		self.javaModelEvaluatorBuilder.load(javaInputStream)
+		javaInputStream.close()
 		return self
 
 	def transform(self, javaPMMLTransformer):
@@ -238,12 +286,12 @@ class LoadingModelEvaluatorBuilder(BaseModelEvaluatorBuilder):
 
 	def transpile(self, path = None):
 		if path:
-			file = self.backend.newObject("java.io.File", path)
-			transpiler = self.backend.newObject("org.jpmml.transpiler.FileTranspiler", None, file)
+			javaFile = self.backend.newObject("java.io.File", path)
+			javaTranspiler = self.backend.newObject("org.jpmml.transpiler.FileTranspiler", None, javaFile)
 		else:
-			transpiler = self.backend.newObject("org.jpmml.transpiler.InMemoryTranspiler", None)
-		transformer = self.backend.newObject("org.jpmml.transpiler.TranspilerTransformer", transpiler)
-		return self.transform(transformer)
+			javaTranspiler = self.backend.newObject("org.jpmml.transpiler.InMemoryTranspiler", None)
+		javaTranspilerTransformer = self.backend.newObject("org.jpmml.transpiler.TranspilerTransformer", javaTranspiler)
+		return self.transform(javaTranspilerTransformer)
 
 def make_backend(alias):
 	if not isinstance(alias, str):
@@ -261,13 +309,14 @@ def make_backend(alias):
 		aliases = ["jpype", "pyjnius", "py4j"]
 		raise ValueError("Java backend alias {0} not in {1}".format(alias, aliases))
 
-def make_evaluator(path, backend = "jpype", lax = False, locatable = False, reporting = False, transpile = False):
+def make_evaluator(obj, backend = "jpype", lax = False, locatable = False, reporting = False, transpile = False):
 	""" Builds an Evaluator based on a PMML file.
 
 	Parameters:
 	----------
-	path: string
-		The path to the PMML file in local filesystem.
+	obj: string or bytes
+		The object to load. Either a path to a PMML file in local filesystem,
+		or a PMML string or byte array.
 
 	backend: JavaBackend or string
 		The Java backend or its alias
@@ -294,24 +343,42 @@ def make_evaluator(path, backend = "jpype", lax = False, locatable = False, repo
 		raise TypeError()
 
 	evaluatorBuilder = LoadingModelEvaluatorBuilder(backend, lax) \
-		.setLocatable(locatable) \
-		.loadFile(path)
+		.setLocatable(locatable)
+
+	if isinstance(obj, Path):
+		evaluatorBuilder = evaluatorBuilder.loadFile(obj)
+	elif isinstance(obj, str):
+		if len(obj) < 1024 and os.path.isfile(obj):
+			evaluatorBuilder = evaluatorBuilder.loadFile(obj)
+		else:
+			evaluatorBuilder = evaluatorBuilder.loadString(obj)
+	elif isinstance(obj, bytes):
+		evaluatorBuilder = evaluatorBuilder.loadBytes(obj)
+	else:
+		raise TypeError()
+
 	if reporting:
-		evaluatorBuilder.setReportingValueFactoryFactory()
+		evaluatorBuilder = evaluatorBuilder.setReportingValueFactoryFactory()
+
 	if transpile:
-		evaluatorBuilder.transpile(transpile if isinstance(transpile, str) else None)
+		evaluatorBuilder = evaluatorBuilder.transpile(transpile if isinstance(transpile, str) else None)
+
 	return evaluatorBuilder.build()
 
-def _package_data_jars(package_data_dir):
+def _package_data_jars(data_dir):
+	package_dir = Path(__import__("jpmml_evaluator").__file__).parent
+	package_data_dir = package_dir / data_dir
+
 	jars = []
-	resources = pkg_resources.resource_listdir(package_data_dir, "")
+	resources = package_data_dir.iterdir()
 	for resource in resources:
-		if resource.endswith(".jar"):
-			jars.append(pkg_resources.resource_filename(package_data_dir, resource))
+		resource = resource.resolve()
+		if resource.suffix == ".jar":
+			jars.append(str(resource))
 	return jars
 
 def _classpath(user_classpath):
 	return _package_classpath() + user_classpath
 
 def _package_classpath():
-	return _package_data_jars("jpmml_evaluator.resources") + _package_data_jars("jpmml_evaluator.dependencies")
+	return _package_data_jars("resources") + _package_data_jars("dependencies")
