@@ -1,4 +1,4 @@
-from jpmml_evaluator import _classpath, JavaError, JNIBackend
+from jpmml_evaluator import _classpath, JavaError, JNIBackend, PythonEvaluatorUtil
 
 class PyJNIusBackend(JNIBackend):
 
@@ -12,7 +12,7 @@ class PyJNIusBackend(JNIBackend):
 		if jnius_config.classpath is None:
 			cls.createJVM()
 		from jnius import autoclass
-		autoclass("org.jpmml.evaluator.python.PythonUtil")
+		autoclass(PythonEvaluatorUtil.JAVA_CLASS_NAME)
 
 	@classmethod
 	def createJVM(cls, user_classpath = []):
@@ -37,6 +37,9 @@ class PyJNIusBackend(JNIBackend):
 		javaClass = self._ensureJavaClass(className)
 		return javaClass(*args)
 
+	def newArray(self, className, values):
+		return list(values)
+
 	def staticInvoke(self, className, methodName, *args):
 		if className == "java.lang.Class" and methodName == "forName":
 			from jnius import find_javaclass
